@@ -1,4 +1,4 @@
-from .helpers import from_client
+from .helpers import from_client, make_neuron_neuroglancer_link
 from .statebuilder import StateBuilder, ChainedStateBuilder
 from .layers import SegmentationLayerConfig, AnnotationLayerConfig, ImageLayerConfig
 from .mappers import (
